@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: mod_security2.c,v 1.11 2006/12/15 15:06:04 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #include <limits.h>
@@ -54,9 +52,25 @@ int perform_interception(modsec_rec *msr) {
     msre_actionset *actionset = NULL;
     const char *message = NULL;
     const char *phase_text = "";
+    const char *intreq_text = "";
+    int is_initial_req = ap_is_initial_req(msr->r);
     int status = DECLINED;
     int log_level = 1;
 
+    /* Check for an initial request */
+
+    if (is_initial_req != 1) {
+        if (msr->r->main != NULL) {
+            intreq_text = "Sub-Request: ";
+        }
+        else if (msr->r->prev != NULL) {
+            intreq_text = "Internal Redirect: ";
+        }
+        else {
+            intreq_text = "Internal Request: ";
+        }
+    }
+
     /* Sanity checks first. */
 
     if (msr->was_intercepted == 0) {
@@ -66,6 +80,7 @@ int perform_interception(modsec_rec *msr) {
 
     if (msr->phase > 4) {
         msr_log(msr, 1, "Internal Error: Asked to intercept request in phase %i.", msr->phase);
+        msr->was_intercepted = 0;
         return DECLINED;
     }
 
@@ -75,12 +90,13 @@ int perform_interception(modsec_rec *msr) {
     phase_text = apr_psprintf(msr->mp, " (phase %i)", msr->phase);
 
     /* By default we log at level 1 but we switch to 4
-     * if a nolog action was used to hide the message.
+     * if a nolog action was used or this is not the initial request
+     * to hide the message.
      */
-    log_level = (actionset->log != 1) ? 4 : 1;
+    log_level = ((actionset->log != 1) || (is_initial_req != 1)) ? 4 : 1;
 
-    /* Pause the request first (if configured to do so). */
-    if (actionset->intercept_pause) {
+    /* Pause the request first (if configured and the initial request). */
+    if (actionset->intercept_pause && (is_initial_req == 1)) {
         msr_log(msr, (log_level > 3 ? log_level : log_level + 1), "Pausing transaction for "
             "%i msec.", actionset->intercept_pause);
         /* apr_sleep accepts microseconds */
@@ -92,14 +108,14 @@ int perform_interception(modsec_rec *msr) {
         case ACTION_DENY :
             if (actionset->intercept_status != 0) {
                 status = actionset->intercept_status;
-                message = apr_psprintf(msr->mp, "Access denied with code %i%s.", status,
-                    phase_text);
+                message = apr_psprintf(msr->mp, "%sAccess denied with code %i%s.",
+                    intreq_text, status, phase_text);
             } else {
                 log_level = 1;
                 status = HTTP_INTERNAL_SERVER_ERROR;
-                message = apr_psprintf(msr->mp, "Access denied with code 500%s "
-                    "(Internal Error: Invalid status code requested %i).", phase_text,
-                    actionset->intercept_status);
+                message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
+                    "(Internal Error: Invalid status code requested %i).",
+                    intreq_text, phase_text, actionset->intercept_status);
             }
             break;
 
@@ -108,23 +124,25 @@ int perform_interception(modsec_rec *msr) {
                 if (ap_find_linked_module("mod_proxy.c") == NULL) {
                     log_level = 1;
                     status = HTTP_INTERNAL_SERVER_ERROR;
-                    message = apr_psprintf(msr->mp, "Access denied with code 500%s "
+                    message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
                         "(Configuration Error: Proxy action to %s requested but mod_proxy not found).",
-                        phase_text, log_escape_nq(msr->mp, actionset->intercept_uri));
+                        intreq_text, phase_text,
+                        log_escape_nq(msr->mp, actionset->intercept_uri));
                 } else {
                     msr->r->filename = apr_psprintf(msr->mp, "proxy:%s", actionset->intercept_uri);
                     msr->r->proxyreq = PROXYREQ_REVERSE;
                     msr->r->handler = "proxy-server";
                     status = OK;
-                    message = apr_psprintf(msr->mp, "Access denied using proxy to %s%s.",
-                        phase_text, log_escape_nq(msr->mp, actionset->intercept_uri));
+                    message = apr_psprintf(msr->mp, "%sAccess denied using proxy to%s %s.",
+                        intreq_text, phase_text,
+                        log_escape_nq(msr->mp, actionset->intercept_uri));
                 }
             } else {
                 log_level = 1;
                 status = HTTP_INTERNAL_SERVER_ERROR;
-                message = apr_psprintf(msr->mp, "Access denied with code 500%s "
+                message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
                     "(Configuration Error: Proxy action requested but it does not work in output phases).",
-                    phase_text);
+                    intreq_text, phase_text);
             }
             break;
 
@@ -141,29 +159,30 @@ int perform_interception(modsec_rec *msr) {
                 if (csd) {
                     if (apr_socket_close(csd) == APR_SUCCESS) {
                         status = HTTP_FORBIDDEN;
-                        message = apr_psprintf(msr->mp, "Access denied with connection close%s.",
-                            phase_text);
+                        message = apr_psprintf(msr->mp, "%sAccess denied with connection close%s.",
+                            intreq_text, phase_text);
                     } else {
                         log_level = 1;
                         status = HTTP_INTERNAL_SERVER_ERROR;
-                        message = apr_psprintf(msr->mp, "Access denied with code 500%s "
+                        message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
                             "(Error: Connection drop requested but failed to close the "
-                            " socket).", phase_text);
+                            " socket).",
+                            intreq_text, phase_text);
                     }
                 } else {
                     log_level = 1;
                     status = HTTP_INTERNAL_SERVER_ERROR;
-                    message = apr_psprintf(msr->mp, "Access denied with code 500%s "
+                    message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
                         "(Error: Connection drop requested but socket not found.",
-                        phase_text);
+                        intreq_text, phase_text);
                 }
             }
             #else
             log_level = 1;
             status = HTTP_INTERNAL_SERVER_ERROR;
-            message = apr_psprintf(msr->mp, "Access denied with code 500%s "
+            message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
                 "(Error: Connection drop not implemented on this platform).",
-                phase_text);
+                intreq_text, phase_text);
             #endif
             break;
 
@@ -176,22 +195,25 @@ int perform_interception(modsec_rec *msr) {
             } else {
                 status = HTTP_MOVED_TEMPORARILY;
             }
-            message = apr_psprintf(msr->mp, "Access denied with redirection to %s using "
-                "status %i%s.", log_escape_nq(msr->mp, actionset->intercept_uri), status,
+            message = apr_psprintf(msr->mp, "%sAccess denied with redirection to %s using "
+                "status %i%s.",
+                intreq_text,
+                log_escape_nq(msr->mp, actionset->intercept_uri), status,
                 phase_text);
             break;
 
         case ACTION_ALLOW :
             status = DECLINED;
-            message = apr_psprintf(msr->mp, "Access allowed%s.", phase_text);
+            message = apr_psprintf(msr->mp, "%sAccess allowed%s.", intreq_text, phase_text);
+            msr->was_intercepted = 0;
             break;
 
         default :
             log_level = 1;
             status = HTTP_INTERNAL_SERVER_ERROR;
-            message = apr_psprintf(msr->mp, "Access denied with code 500%s "
+            message = apr_psprintf(msr->mp, "%sAccess denied with code 500%s "
                 "(Internal Error: invalid interception action %i).",
-                phase_text, actionset->intercept_action);
+                intreq_text, phase_text, actionset->intercept_action);
             break;
     }
 
@@ -205,13 +227,14 @@ int perform_interception(modsec_rec *msr) {
  * Retrieves a previously stored transaction context by
  * looking at the main request, and the previous requests.
  */
-static modsec_rec *retrieve_tx_context(const request_rec *r) {
+static modsec_rec *retrieve_tx_context(request_rec *r) {
     modsec_rec *msr = NULL;
     request_rec *rx = NULL;
 
     /* Look in the current request first. */
     msr = (modsec_rec *)apr_table_get(r->notes, NOTE_MSR);
     if (msr != NULL) {
+        msr->r = r;
         return msr;
     }
 
@@ -219,6 +242,7 @@ static modsec_rec *retrieve_tx_context(const request_rec *r) {
     if (r->main != NULL) {
         msr = (modsec_rec *)apr_table_get(r->main->notes, NOTE_MSR);
         if (msr != NULL) {
+            msr->r = r;
             return msr;
         }
     }
@@ -228,6 +252,7 @@ static modsec_rec *retrieve_tx_context(const request_rec *r) {
     while(rx != NULL) {
         msr = (modsec_rec *)apr_table_get(rx->notes, NOTE_MSR);
         if (msr != NULL) {
+            msr->r = r;
             return msr;
         }
         rx = rx->prev;
@@ -329,6 +354,8 @@ static modsec_rec *create_tx_context(request_rec *r) {
         msr_log(msr, 4, "Transaction context created (dcfg %x).", msr->dcfg1);
     }
 
+    msr->msc_rule_mptmp = NULL;
+
     return msr;    
 }
 
@@ -428,7 +455,7 @@ static int hook_post_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_t
 
         if (first_time == 0) {
             ap_log_error(APLOG_MARK, APLOG_NOTICE | APLOG_NOERRNO, 0, s,
-                "ModSecurity: chroot checkpoint #2 (pid=%i ppid=%i)", getpid(), getppid());
+                "ModSecurity: chroot checkpoint #2 (pid=%d ppid=%d)", getpid(), getppid());
 
             if (chdir(chroot_dir) < 0) {
                 ap_log_error(APLOG_MARK, APLOG_ERR | APLOG_NOERRNO, 0, s,
@@ -457,7 +484,7 @@ static int hook_post_config(apr_pool_t *mp, apr_pool_t *mp_log, apr_pool_t *mp_t
                 "ModSecurity: chroot successful, path=%s", chroot_dir);
         } else {
             ap_log_error(APLOG_MARK, APLOG_NOTICE | APLOG_NOERRNO, 0, s,
-                "ModSecurity: chroot checkpoint #1 (pid=%i ppid=%i)", getpid(), getppid());
+                "ModSecurity: chroot checkpoint #1 (pid=%d ppid=%d)", getpid(), getppid());
         }
     }
     #endif
@@ -527,7 +554,7 @@ static int hook_request_early(request_rec *r) {
 
     /* Process phase REQUEST_HEADERS */
     rc = DECLINED;
-    if (modsecurity_process_phase(msr, PHASE_REQUEST_HEADERS)) {
+    if (modsecurity_process_phase(msr, PHASE_REQUEST_HEADERS) > 0) {
         rc = perform_interception(msr);
     }
 
@@ -553,18 +580,34 @@ static int hook_request_late(request_rec *r) {
          */
         return DECLINED;
     }
-    msr->r = r;
-    msr->remote_user = r->user;
 
     /* Has this phase been completed already? */
     if (msr->phase_request_body_complete) {
+        /* If we are redirecting and there was no previous response it is
+         * an error page request and we ignore it.
+         */
+        if (    (msr->r->prev != NULL)
+            && ((msr->r->prev->headers_out == NULL) || (apr_is_empty_table(msr->r->prev->headers_out))) )
+        {
+            msr_log(msr, 9, "Allowing internally redirected error document: %s", msr->r->uri);
+            return DECLINED;
+        }
+
+        if (msr->was_intercepted) {
+            msr_log(msr, 4, "Phase REQUEST_BODY request already intercepted. Intercepting additional request.");
+            return perform_interception(msr);
+        }
+
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Phase REQUEST_BODY already complete, skipping.");
         }
+
         return DECLINED;
     }
     msr->phase_request_body_complete = 1;
 
+    msr->remote_user = r->user;
+
     /* Get the second configuration context. */
     msr->dcfg2 = (directory_config *)ap_get_module_config(r->per_dir_config,
         &security2_module);    
@@ -636,7 +679,7 @@ static int hook_request_late(request_rec *r) {
     record_time_checkpoint(msr, 1);
 
     rc = DECLINED;
-    if (modsecurity_process_phase(msr, PHASE_REQUEST_BODY)) {
+    if (modsecurity_process_phase(msr, PHASE_REQUEST_BODY) > 0) {
         rc = perform_interception(msr);
     }
 
@@ -655,7 +698,24 @@ static void hook_error_log(const char *file, int line, int level, apr_status_t s
     error_message *em = NULL;
 
     if (r == NULL) return;
-    msr = retrieve_tx_context(r);
+    msr = retrieve_tx_context((request_rec *)r);
+
+    /* Create a context for requests we never had the chance to process */ 
+    if ((msr == NULL)
+        && ((level & APLOG_LEVELMASK) < APLOG_DEBUG)
+        && apr_table_get(r->subprocess_env, "UNIQUE_ID"))
+    {
+        msr = create_tx_context((request_rec *)r); 
+        if (msr->txcfg->debuglog_level >= 9) {
+            if (msr == NULL) {
+                msr_log(msr, 9, "Failed to create context after request failure.");
+            }
+            else {
+                msr_log(msr, 9, "Context created after request failure.");
+            }
+        }
+    } 
+
     if (msr == NULL) return;
 
     /* Store the error message for later */
@@ -828,23 +888,25 @@ static int hook_log_transaction(request_rec *r) {
 static void hook_insert_filter(request_rec *r) {
     modsec_rec *msr = NULL;
 
-    /* Find the transaction context and make sure we are
-     * supposed to proceed.
-     */
+    /* Find the transaction context first. */
     msr = retrieve_tx_context(r);
     if (msr == NULL) return;
 
+    /* Only proceed to add the filter if the engine is enabled. */
     if (msr->txcfg->is_enabled == 0) {
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Hook insert_filter: Processing disabled, skipping.");
         }
+
         return;
     }
 
+    /* Add the input filter, but only if we need it to run. */
     if (msr->if_status == IF_STATUS_WANTS_TO_RUN) {
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Hook insert_filter: Adding input forwarding filter (r %x).", r);
         }
+
         ap_add_input_filter("MODSECURITY_IN", msr, r, r->connection);
     }
 
@@ -856,6 +918,7 @@ static void hook_insert_filter(request_rec *r) {
         if (msr->txcfg->debuglog_level >= 4) {
             msr_log(msr, 4, "Hook insert_filter: Adding output filter (r %x).", r);
         }
+
         ap_add_output_filter("MODSECURITY_OUT", msr, r, r->connection);
     }
 }
@@ -899,6 +962,7 @@ static void hook_insert_error_filter(request_rec *r) {
 }
 #endif
 
+#if (!defined(NO_MODSEC_API))
 /**
  * This function is exported for other Apache modules to
  * register new transformation functions.
@@ -918,6 +982,7 @@ static void modsec_register_operator(const char *name, void *fn_init, void *fn_e
         msre_engine_op_register(modsecurity->msre, name, fn_init, fn_exec);
     }
 }
+#endif
 
 /**
  * Registers module hooks with Apache.
@@ -934,6 +999,10 @@ static void register_hooks(apr_pool_t *mp) {
         NULL
     };
     static const char *postread_beforeme_list[] = {
+        "mod_rpaf.c",
+        "mod_extract_forwarded2.c",
+        "mod_breach_realip.c",
+        "mod_breach_trans.c",
         "mod_unique_id.c",
         NULL
     };
@@ -942,9 +1011,11 @@ static void register_hooks(apr_pool_t *mp) {
         NULL
     };
 
+#if (!defined(NO_MODSEC_API))
     /* Export optional functions. */
     APR_REGISTER_OPTIONAL_FN(modsec_register_tfn);
     APR_REGISTER_OPTIONAL_FN(modsec_register_operator);
+#endif
 
     /* Main hooks */
     ap_hook_pre_config(hook_pre_config, NULL, NULL, APR_HOOK_FIRST);
