@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: modsecurity.h,v 1.27 2007/02/05 12:44:40 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #ifndef _MODSECURITY_H_
@@ -26,7 +24,7 @@ typedef struct msc_data_chunk msc_data_chunk;
 typedef struct msc_arg msc_arg;
 typedef struct msc_string msc_string;
 
-#if !(defined(WIN32) || defined(NETWARE))
+#if !(defined(WIN32) || defined(CYGWIN) || defined(NETWARE) || defined(SOLARIS2))
 #define DSOLOCAL __attribute__((visibility("hidden")))
 #else
 #define DSOLOCAL
@@ -50,7 +48,7 @@ typedef struct msc_string msc_string;
 #include "http_protocol.h"
 
 #define MODULE_NAME "ModSecurity"
-#define MODULE_RELEASE "2.1.1"
+#define MODULE_RELEASE "2.1.3"
 #define MODULE_NAME_FULL (MODULE_NAME " v" MODULE_RELEASE " (Apache 2.x)")
 
 #define PHASE_REQUEST_HEADERS       1
@@ -306,6 +304,9 @@ struct modsec_rec {
     /* other */
     apr_table_t         *collections;
     apr_table_t         *collections_dirty;
+
+    /* rule processing temp pool */
+    apr_pool_t          *msc_rule_mptmp;
 };
 
 struct directory_config {
@@ -346,9 +347,12 @@ struct directory_config {
      * name of the index file (for the new audit log type)
      */
     char                *auditlog_name;
+    /* The name of the secondary index file */
+    char                *auditlog2_name;
 
-    /* The file descriptor for the file above */
+    /* The file descriptors for the files above */
     apr_file_t          *auditlog_fd;
+    apr_file_t          *auditlog2_fd;
 
     /* For the new-style audit log only, the path where
      * audit log entries will be stored
