@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: msc_logging.c,v 1.1.1.1 2006/10/14 09:30:43 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #include "msc_logging.h"
@@ -886,5 +884,16 @@ void sec_audit_logger(modsec_rec *msr) {
     if (text == NULL) return;
 
     nbytes = strlen(text);
+    if (msr->txcfg->debuglog_level >= 9) {
+        msr_log(msr, 9, "Audit Log: Writing %d bytes to primary concurrent index", nbytes);
+    }
     apr_file_write_full(msr->txcfg->auditlog_fd, text, nbytes, &nbytes_written);
+
+    /* Write to the secondary audit log if we have one */
+    if (msr->txcfg->auditlog2_fd != NULL) {
+        if (msr->txcfg->debuglog_level >= 9) {
+            msr_log(msr, 9, "Audit Log: Writing %d bytes to secondary concurrent index", nbytes);
+        }
+        apr_file_write_full(msr->txcfg->auditlog2_fd, text, nbytes, &nbytes_written);
+    }
 }
