@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: msc_multipart.c,v 1.2 2006/10/16 04:41:51 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #include <ctype.h>
@@ -174,38 +172,67 @@ static int multipart_parse_content_disposition(modsec_rec *msr, char *c_d_value)
  *
  */
 static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
-    int rc;
+    int i, len, rc;
     
     if (error_msg == NULL) return -1;
     *error_msg = NULL;
 
-    if ((msr->mpd->buf[0] == '\r')
-        &&(msr->mpd->buf[1] == '\n')
-        &&(msr->mpd->buf[2] == '\0'))
-    {
-        char *header_value;
-        
-        /* empty line */
+    /* Check for nul bytes. */
+    len = MULTIPART_BUF_SIZE - msr->mpd->bufleft;
+    for(i = 0; i < len; i++) {
+        if (msr->mpd->buf[i] == '\0') {
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Nul byte in part headers.");
+            return -1;
+        }        
+    }
+
+    if (len > 1) {
+        if (msr->mpd->buf[len - 2] == '\r') {
+            msr->mpd->flag_crlf_line = 1;
+        } else {
+            msr->mpd->flag_lf_line = 1;
+        }
+    } else {
+        msr->mpd->flag_lf_line = 1;
+    }
+
+    /* Is this an empty line? */
+    if (   ((msr->mpd->buf[0] == '\r')
+          &&(msr->mpd->buf[1] == '\n')
+          &&(msr->mpd->buf[2] == '\0') )
+        || ((msr->mpd->buf[0] == '\n')
+          &&(msr->mpd->buf[1] == '\0') ) )
+    { /* Empty line. */
+        char *header_value = NULL;
 
         header_value = (char *)apr_table_get(msr->mpd->mpp->headers, "Content-Disposition");
         if (header_value == NULL) {
-            *error_msg = apr_psprintf(msr->mp, "Multipart: Part is missing the Content-Disposition header");
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Part missing Content-Disposition header.");
             return -1;
         }
         
         rc = multipart_parse_content_disposition(msr, header_value);
         if (rc < 0) {
-            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid Content-Disposition header (%i): %s",
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid Content-Disposition header (%i): %s.",
                 rc, log_escape_nq(msr->mp, header_value));
             return -1;
         }
 
         if (msr->mpd->mpp->name == NULL) {
-            *error_msg = apr_psprintf(msr->mp, "Multipart: Part name missing");
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Content-Disposition header missing name field.");
             return -1;
-        }        
-        
+        }
+
         if (msr->mpd->mpp->filename != NULL) {
+            /* Some parsers use crude methods to extract the name and filename
+             * values from the C-D header. We need to check for the case where they
+             * didn't understand C-D but we did.
+             */
+            if (strstr(header_value, "filename=") == NULL) {
+                *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid Content-Disposition header (filename).");
+                return -1;
+            }
+
             msr->mpd->mpp->type = MULTIPART_FILE;
         } else {
             msr->mpd->mpp->type = MULTIPART_FORMDATA;
@@ -214,19 +241,21 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
         msr->mpd->mpp_state = 1;
         msr->mpd->mpp->last_header_name = NULL;
     } else {
-        /* header line */
+        /* Header line. */
+
         if ((msr->mpd->buf[0] == '\t')||(msr->mpd->buf[0] == ' ')) {
             char *header_value, *new_value, *data;
             
             /* header folding, add data to the header we are building */
+            msr->mpd->flag_header_folding = 1;
             
             if (msr->mpd->mpp->last_header_name == NULL) {
                 /* we are not building a header at this moment */
-                *error_msg = apr_psprintf(msr->mp, "Multipart: invalid part header (invalid folding)");
+                *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid part header (folding error).");
                 return -1;
             }
             
-            /* locate the beginning of the data */
+            /* locate the beginning of data */
             data = msr->mpd->buf;
             while((*data == '\t')||(*data == ' ')) data++;
             
@@ -243,7 +272,7 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
                 log_escape(msr->mp, data));
             
             if (strlen(new_value) > 4096) {
-                *error_msg = apr_psprintf(msr->mp, "Multpart: invalid part header (too long)");
+                *error_msg = apr_psprintf(msr->mp, "Multipart: Part header too long.");
                 return -1;
             }
         } else {
@@ -254,7 +283,7 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
             data = msr->mpd->buf;
             while((*data != ':')&&(*data != '\0')) data++;
             if (*data == '\0') {
-                *error_msg = apr_psprintf(msr->mp, "Multipart: invalid part header (missing colon): %s",
+                *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid part header (colon missing): %s.",
                     log_escape_nq(msr->mp, msr->mpd->buf));
                 return -1;
             }
@@ -269,7 +298,7 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
                         
             /* error if the name already exists */
             if (apr_table_get(msr->mpd->mpp->headers, header_name) != NULL) {
-                *error_msg = apr_psprintf(msr->mp, "Multipart: part header already exists: %s",
+                *error_msg = apr_psprintf(msr->mp, "Multipart: Duplicate part header: %s.",
                     log_escape_nq(msr->mp, header_name));
                 return -1;
             }
@@ -290,22 +319,36 @@ static int multipart_process_part_header(modsec_rec *msr, char **error_msg) {
  *
  */
 static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
-    char *p = msr->mpd->buf + (MULTIPART_BUF_SIZE - msr->mpd->bufleft) - 2;
+    char *p = msr->mpd->buf + (MULTIPART_BUF_SIZE - msr->mpd->bufleft);
     char localreserve[2] = { '\0', '\0' }; /* initialized to quiet warning */
     int bytes_reserved = 0;
     
     if (error_msg == NULL) return -1;
     *error_msg = NULL;
-    
-    /* preserve the last two bytes for later */
-    if (MULTIPART_BUF_SIZE - msr->mpd->bufleft >= 2) {
-        bytes_reserved = 1;
-        localreserve[0] = *p;
-        localreserve[1] = *(p + 1);
-        msr->mpd->bufleft += 2;
-        *p = 0;
-    }
 
+    /* Preserve some bytes for later. */
+    if (   ((MULTIPART_BUF_SIZE - msr->mpd->bufleft) >= 1)
+        && (*(p - 1) == '\n') )
+    {
+        if (   ((MULTIPART_BUF_SIZE - msr->mpd->bufleft) >= 2)
+            && (*(p - 2) == '\r') )
+        {
+            /* Two bytes. */
+            bytes_reserved = 2;
+            localreserve[0] = *(p - 2);
+            localreserve[1] = *(p - 1);
+            msr->mpd->bufleft += 2;
+            *(p - 2) = 0;
+        } else {
+            /* Only one byte. */
+            bytes_reserved = 1;
+            localreserve[0] = *(p - 1);
+            localreserve[1] = 0;
+            msr->mpd->bufleft += 1;
+            *(p - 1) = 0;
+        }
+    }
+    
     /* add data to the part we are building */
     if (msr->mpd->mpp->type == MULTIPART_FILE) {
 
@@ -320,8 +363,6 @@ static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
         if (msr->upload_extract_files) {
             /* first create a temporary file if we don't have it already */
             if (msr->mpd->mpp->tmp_file_fd == 0) {
-                // char *filename = multipart_construct_filename(msr);
-
                 /* construct temporary file name */
                 msr->mpd->mpp->tmp_file_name = apr_psprintf(msr->mp, "%s/%s-%s-file-XXXXXX",
                     msr->txcfg->tmp_dir, current_filetime(msr->mp), msr->txid);
@@ -339,14 +380,15 @@ static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
             }
 
             /* write the reserve first */
-            if (msr->mpd->reserve[0] == 1) {
-                if (write(msr->mpd->mpp->tmp_file_fd, &msr->mpd->reserve[1], 2) != 2) {
+            if (msr->mpd->reserve[0] != 0) {
+                if (write(msr->mpd->mpp->tmp_file_fd, &msr->mpd->reserve[1], msr->mpd->reserve[0]) != msr->mpd->reserve[0]) {
                     *error_msg = apr_psprintf(msr->mp, "Multipart: writing to \"%s\" failed",
                         log_escape(msr->mp, msr->mpd->mpp->tmp_file_name));
                     return -1;
                 }
-                msr->mpd->mpp->tmp_file_size += 2;
-                msr->mpd->mpp->length += 2;
+
+                msr->mpd->mpp->tmp_file_size += msr->mpd->reserve[0];
+                msr->mpd->mpp->length += msr->mpd->reserve[0];
             }
 
             /* write data to the file */
@@ -362,11 +404,8 @@ static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
             msr->mpd->mpp->length += (MULTIPART_BUF_SIZE - msr->mpd->bufleft);
         } else {
             /* just keep track of the file size */
-            if (msr->mpd->reserve[0] == 1) {
-                msr->mpd->mpp->tmp_file_size += 2;
-            }
-            msr->mpd->mpp->tmp_file_size += (MULTIPART_BUF_SIZE - msr->mpd->bufleft);
-            msr->mpd->mpp->length += (MULTIPART_BUF_SIZE - msr->mpd->bufleft);
+            msr->mpd->mpp->tmp_file_size += (MULTIPART_BUF_SIZE - msr->mpd->bufleft) + msr->mpd->reserve[0];
+            msr->mpd->mpp->length += (MULTIPART_BUF_SIZE - msr->mpd->bufleft)  + msr->mpd->reserve[0];
         }
     }
     else if (msr->mpd->mpp->type == MULTIPART_FORMDATA) {
@@ -378,12 +417,13 @@ static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
         if (msr->mpd->mpp->length == 0) {
             msr->mpd->mpp->offset = msr->mpd->buf_offset;
         }
-        
-        if (msr->mpd->reserve[0] == 1) {
-            value_part->data = apr_palloc(msr->mp, (MULTIPART_BUF_SIZE - msr->mpd->bufleft) + 2);
-            memcpy(value_part->data, &(msr->mpd->reserve[1]), 2);
-            memcpy(value_part->data + 2, msr->mpd->buf, (MULTIPART_BUF_SIZE - msr->mpd->bufleft));
-            value_part->length = (MULTIPART_BUF_SIZE - msr->mpd->bufleft) + 2;
+
+        if (msr->mpd->reserve[0] != 0) {
+            value_part->data = apr_palloc(msr->mp, (MULTIPART_BUF_SIZE - msr->mpd->bufleft) + msr->mpd->reserve[0]);
+            memcpy(value_part->data, &(msr->mpd->reserve[1]), msr->mpd->reserve[0]);
+            memcpy(value_part->data + msr->mpd->reserve[0], msr->mpd->buf, (MULTIPART_BUF_SIZE - msr->mpd->bufleft));
+
+            value_part->length = (MULTIPART_BUF_SIZE - msr->mpd->bufleft) + msr->mpd->reserve[0];
             msr->mpd->mpp->length += value_part->length;
         } else {
             value_part->length = (MULTIPART_BUF_SIZE - msr->mpd->bufleft);
@@ -405,14 +445,14 @@ static int multipart_process_part_data(modsec_rec *msr, char **error_msg) {
      * context so that they don't get lost
      */
     if (bytes_reserved) {
-        msr->mpd->reserve[0] = 1;
+        msr->mpd->reserve[0] = bytes_reserved;
         msr->mpd->reserve[1] = localreserve[0];
         msr->mpd->reserve[2] = localreserve[1];
-        msr->mpd->buf_offset += 2;
+        msr->mpd->buf_offset += bytes_reserved;
     }
     else {
+        msr->mpd->buf_offset -= msr->mpd->reserve[0];
         msr->mpd->reserve[0] = 0;
-        msr->mpd->buf_offset -= 2;
     }
     
     return 1;
@@ -501,6 +541,74 @@ static int multipart_process_boundary(modsec_rec *msr, int last_part, char **err
     return 1;
 }
 
+static int multipart_boundary_characters_valid(char *boundary) {
+    unsigned char *p = (unsigned char *)boundary;
+    unsigned char c;
+
+    if (p == NULL) return -1;
+
+    while((c = *p) != '\0') {
+        /* Control characters and space not allowed. */
+        if (c < 32) {
+            return 0;
+        }
+
+        /* Non-ASCII characters not allowed. */
+        if (c > 126) {
+            return 0;
+        }
+
+        switch(c) {
+            /* Special characters not allowed. */
+            case '(' :
+            case ')' :
+            case '<' :
+            case '>' :
+            case '@' :
+            case ',' :
+            case ';' :
+            case ':' :
+            case '\\' :
+            case '"' :
+            case '/' :
+            case '[' :
+            case ']' :
+            case '?' :
+            case '=' :
+                return 0;
+                break;
+            
+            default :
+                /* Do nothing. */
+                break;
+        }
+
+        p++;
+    }
+
+    return 1;
+}
+
+static int multipart_count_boundary_params(apr_pool_t *mp, const char *header_value) {
+    char *duplicate = NULL;
+    char *s = NULL;
+    int count = 0;
+
+    if (header_value == NULL) return -1;
+    duplicate = apr_pstrdup(mp, header_value);
+    if (duplicate == NULL) return -1;
+
+    /* Performing a case-insensitive search. */
+    strtolower_inplace((unsigned char *)duplicate);
+
+    s = duplicate;
+    while((s = strstr(s, "boundary")) != NULL) {
+        count++;
+        s += 8;
+    }
+
+    return count;
+}
 
 /**
  *
@@ -512,26 +620,154 @@ int multipart_init(modsec_rec *msr, char **error_msg) {
     msr->mpd = (multipart_data *)apr_pcalloc(msr->mp, sizeof(multipart_data));
     if (msr->mpd == NULL) return -1;
 
-    if (msr->request_content_type == NULL) {
-        *error_msg = apr_psprintf(msr->mp, "Multipart: Content-Type header not available.");
-        return -1;
-    }
-
-    msr->mpd->boundary = strstr(msr->request_content_type, "boundary=");
-    if ((msr->mpd->boundary != NULL)&&(*(msr->mpd->boundary + 9) != 0)) {
-        msr->mpd->boundary = msr->mpd->boundary + 9;
-    }
-    else {
-        *error_msg = apr_psprintf(msr->mp, "Multipart Boundary not found or invalid.");
-        return -1;
-    }
-
     msr->mpd->parts = apr_array_make(msr->mp, 10, sizeof(multipart_part *));
     msr->mpd->bufleft = MULTIPART_BUF_SIZE;
     msr->mpd->bufptr = msr->mpd->buf;
     msr->mpd->buf_contains_line = 1;
     msr->mpd->mpp = NULL;
 
+    if (msr->request_content_type == NULL) {
+        msr->mpd->flag_error = 1;
+        *error_msg = apr_psprintf(msr->mp, "Multipart: Content-Type header not available.");
+        return -1;
+    }
+
+    if (strlen(msr->request_content_type) > 1024) {
+        msr->mpd->flag_error = 1;
+        *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (length).");
+        return -1;
+    }
+
+    if (strncasecmp(msr->request_content_type, "multipart/form-data", 19) != 0) {
+        msr->mpd->flag_error = 1;
+        *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid MIME type.");
+        return -1;
+    }
+
+    /* Count how many times the word "boundary" appears in the C-T header. */
+    if (multipart_count_boundary_params(msr->mp, msr->request_content_type) > 1) {
+        msr->mpd->flag_error = 1;
+        *error_msg = apr_psprintf(msr->mp, "Multipart: Multiple boundary parameters in C-T.");
+        return -1;
+    }
+    
+    msr->mpd->boundary = strstr(msr->request_content_type, "boundary");
+    if (msr->mpd->boundary != NULL) {
+        char *p = NULL;
+        char *b = NULL;
+        int seen_semicolon = 0;
+        int len = 0;
+
+        /* Check for extra characters before the boundary. */
+        for (p = (char *)(msr->request_content_type + 19); p < msr->mpd->boundary; p++) {
+            if (!isspace(*p)) {
+                if ((seen_semicolon == 0)&&(*p == ';')) {
+                    seen_semicolon = 1; /* It is OK to have one semicolon. */
+                } else {
+                    msr->mpd->flag_error = 1;
+                    *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (malformed).");
+                    return -1;
+                }
+            }
+        }
+
+        /* Have we seen the semicolon in the header? */
+        if (seen_semicolon == 0) {
+            msr->mpd->flag_missing_semicolon = 1;
+        }
+        
+        b = strchr(msr->mpd->boundary + 8, '=');
+        if (b == NULL) {
+            msr->mpd->flag_error = 1;
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (malformed).");
+            return -1;
+        }
+
+        /* Check parameter name ends well. */
+        if (b != (msr->mpd->boundary + 8)) {
+            /* Check all characters between the end of the boundary
+             * and the = character.
+             */
+            for (p = msr->mpd->boundary + 8; p < b; p++) {
+                if (isspace(*p)) {
+                    /* Flag for whitespace after parameter name. */
+                    msr->mpd->flag_boundary_whitespace = 1;
+                } else {
+                    msr->mpd->flag_error = 1;
+                    *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (parameter name).");
+                    return -1;
+                }
+            }
+        }
+        
+        b++; /* Go over the = character. */
+        len = strlen(b);
+        
+        /* Flag for whitespace before parameter value. */
+        if (isspace(*b)) {
+            msr->mpd->flag_boundary_whitespace = 1;
+        }
+
+        /* Is the boundary quoted? */
+        if ((len >= 2)&&(*b == '"')&&(*(b + len - 1) == '"')) {
+            /* Quoted. */
+            msr->mpd->boundary = apr_pstrndup(msr->mp, b + 1, len - 2);
+            if (msr->mpd->boundary == NULL) return -1;
+            msr->mpd->flag_boundary_quoted = 1;
+        } else {
+            /* Not quoted. */
+
+            /* Test for partial quoting. */
+            if (   (*b == '"')
+                || ((len >= 2)&&(*(b + len - 1) == '"')) )
+            {
+                msr->mpd->flag_error = 1;
+                *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (quote).");
+                return -1;
+            }
+
+            msr->mpd->boundary = apr_pstrdup(msr->mp, b);
+            if (msr->mpd->boundary == NULL) return -1;
+            msr->mpd->flag_boundary_quoted = 0;
+        }
+
+        /* Case-insensitive test for the string "boundary" in the boundary. */
+        if (multipart_count_boundary_params(msr->mp, msr->mpd->boundary) != 0) {
+            msr->mpd->flag_error = 1;
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (content).");
+            return -1;
+        }
+
+        /* Validate the characters used in the boundary. */
+        if (multipart_boundary_characters_valid(msr->mpd->boundary) != 1) {
+            msr->mpd->flag_error = 1;
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (characters).");
+            return -1;
+        }
+
+        msr_log(msr, 9, "Multipart: Boundary%s: %s",
+            (msr->mpd->flag_boundary_quoted ? " (quoted)" : ""),
+            log_escape_nq(msr->mp, msr->mpd->boundary));
+
+        if (strlen(msr->mpd->boundary) == 0) {
+            msr->mpd->flag_error = 1;
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (empty).");
+            return -1;
+        }
+    }
+    else { /* Could not find boundary in the C-T header. */
+        msr->mpd->flag_error = 1;
+
+        /* Test for case-insensitive boundary. Allowed by the RFC but highly unusual. */
+        if (multipart_count_boundary_params(msr->mp, msr->request_content_type) > 0) {
+            *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary in C-T (case sensitivity).");
+            return -1;
+        }
+
+        *error_msg = apr_psprintf(msr->mp, "Multipart: Boundary not found in C-T.");
+        return -1;
+    }
+
     return 1;
 }
 
@@ -542,7 +778,11 @@ int multipart_complete(modsec_rec *msr, char **error_log) {
     if (msr->mpd == NULL) return 1;
 
     if ((msr->mpd->seen_data != 0)&&(msr->mpd->is_complete == 0)) {
-        *error_log = apr_psprintf(msr->mp, "Multipart: final boundary missing");
+        if (msr->mpd->boundary_count > 0) {
+            *error_log = apr_psprintf(msr->mp, "Multipart: Final boundary missing.");
+        } else {
+            *error_log = apr_psprintf(msr->mp, "Multipart: No boundaries found in payload.");
+        }
         return -1;
     }
 
@@ -563,20 +803,22 @@ int multipart_process_chunk(modsec_rec *msr, const char *buf,
 
     if (size == 0) return 1;
 
-    if (msr->mpd->seen_data == 0) msr->mpd->seen_data = 1;
+    msr->mpd->seen_data = 1;
     
     if (msr->mpd->is_complete) {
+        msr->mpd->flag_data_before = 1;
         msr_log(msr, 4, "Multipart: Ignoring data after last boundary (received %i bytes)", size);
         return 1;
     }
     
     if (msr->mpd->bufleft == 0) {
+        msr->mpd->flag_error = 1;
         *error_msg = apr_psprintf(msr->mp,
             "Multipart: Internal error in process_chunk: no space left in the buffer");
         return -1;
     }
 
-    /* here we loop through the data available, byte by byte */
+    /* here we loop through the available data, one byte at a time */
     while(inleft > 0) {
         char c = *inptr;
         int process_buffer = 0;
@@ -597,47 +839,122 @@ int multipart_process_chunk(modsec_rec *msr, const char *buf,
          * or the end of our internal buffer
          */
         if ((c == 0x0a)||(msr->mpd->bufleft == 0)||(process_buffer)) {
+            int processed_as_boundary = 0;
+
             *(msr->mpd->bufptr) = 0;
 
-            /* boundary preconditions: length of the line greater than
-             * the length of the boundary + the first two characters
-             * are dashes "-"
-             */
-            if ( msr->mpd->buf_contains_line
-                && (strlen(msr->mpd->buf) > strlen(msr->mpd->boundary) + 2)
-                && (((*(msr->mpd->buf) == '-'))&&(*(msr->mpd->buf + 1) == '-'))
-                && (strncmp(msr->mpd->buf + 2, msr->mpd->boundary, strlen(msr->mpd->boundary)) == 0) ) {
-                
-                char *boundary_end = msr->mpd->buf + 2 + strlen(msr->mpd->boundary);
-                    
-                if (  (*boundary_end == '\r')
-                    &&(*(boundary_end + 1) == '\n')
-                    &&(*(boundary_end + 2) == '\0')
-                ) {
-                    /* simple boundary */
-                    if (multipart_process_boundary(msr, 0, error_msg) < 0) return -1;
+            /* Do we have something that looks like a boundary? */
+            if (msr->mpd->buf_contains_line
+                && (strlen(msr->mpd->buf) > 3)
+                && (((*(msr->mpd->buf) == '-'))&&(*(msr->mpd->buf + 1) == '-')) )
+            {
+                /* Does it match our boundary? */
+                if ((strlen(msr->mpd->buf) >= strlen(msr->mpd->boundary) + 2)
+                    && (strncmp(msr->mpd->buf + 2, msr->mpd->boundary, strlen(msr->mpd->boundary)) == 0) )
+                {
+                    char *boundary_end = msr->mpd->buf + 2 + strlen(msr->mpd->boundary);
+                    int is_final = 0;
+
+                    /* Is this the final boundary? */                
+                    if ((*boundary_end == '-')&&(*(boundary_end + 1)== '-')) {
+                        is_final = 1;
+                        boundary_end += 2;
+
+                        if (msr->mpd->is_complete != 0) {
+                            msr->mpd->flag_error = 1;
+                            *error_msg = apr_psprintf(msr->mp,
+                                "Multipart: Invalid boundary (final duplicate).");
+                            return -1;
+                        }
+                    }
+
+                    /* Allow for CRLF and LF line endings. */
+                    if (   ( (*boundary_end == '\r')
+                              && (*(boundary_end + 1) == '\n')
+                              && (*(boundary_end + 2) == '\0') )
+                        || ( (*boundary_end == '\n')
+                              && (*(boundary_end + 1) == '\0') ) )
+                    {
+                        if (*boundary_end == '\n') {
+                            msr->mpd->flag_lf_line = 1;
+                        } else {
+                            msr->mpd->flag_crlf_line = 1;
+                        }
+
+                        if (multipart_process_boundary(msr, (is_final ? 1 : 0), error_msg) < 0) {
+                            msr->mpd->flag_error = 1;
+                            return -1;
+                        }
+
+                        if (is_final) {
+                            msr->mpd->is_complete = 1;
+                        }
+
+                        processed_as_boundary = 1;
+                        msr->mpd->boundary_count++;
+                    }
+                    else {
+                        /* error */
+                        msr->mpd->flag_error = 1;
+                        *error_msg = apr_psprintf(msr->mp,
+                            "Multipart: Invalid boundary: %s",
+                            log_escape_nq(msr->mp, msr->mpd->buf));
+                        return -1;
+                    }
+                } else { /* It looks like a boundary but we couldn't match it. */
+                    char *p = NULL;
+
+                    /* Check if an attempt to use quotes around the boundary was made. */
+                    if (   (msr->mpd->flag_boundary_quoted)
+                        && (strlen(msr->mpd->buf) >= strlen(msr->mpd->boundary) + 3)
+                        && (*(msr->mpd->buf + 2) == '"')
+                        && (strncmp(msr->mpd->buf + 3, msr->mpd->boundary, strlen(msr->mpd->boundary)) == 0)
+                    ) {
+                        msr->mpd->flag_error = 1;
+                        *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary (quotes).");
+                        return -1;
+                    }
+
+                    /* Check the beginning of the boundary for whitespace. */
+                    p = msr->mpd->buf + 2;
+                    while(isspace(*p)) {
+                        p++;
+                    }
+    
+                    if (   (p != msr->mpd->buf + 2)
+                        && (strncmp(p, msr->mpd->boundary, strlen(msr->mpd->boundary)))
+                    ) {
+                        /* Found whitespace in front of a boundary. */
+                        msr->mpd->flag_error = 1;
+                        *error_msg = apr_psprintf(msr->mp, "Multipart: Invalid boundary (whitespace).");
+                        return -1;
+                    }
+
+                    msr->mpd->flag_unmatched_boundary = 1;
                 }
-                else
-                if (  (*boundary_end == '-')
-                    &&(*(boundary_end + 1) == '-')
-                    &&(*(boundary_end + 2) == '\r')
-                    &&(*(boundary_end + 3) == '\n')
-                    &&(*(boundary_end + 4) == '\0')
-                ) {
-                    /* final boundary */
-                    msr->mpd->is_complete = 1;
-                    if (multipart_process_boundary(msr, 1, error_msg) < 0) return -1;
-                }
-                else {
-                    /* error */
-                    *error_msg = apr_psprintf(msr->mp,
-                        "Multipart: Invalid boundary detected: %s",
-                        log_escape_nq(msr->mp, msr->mpd->buf));
-                    return -1;
+            } else { /* We do not think the buffer contains a boundary. */
+                /* Look into the buffer to see if there's anything
+                 * there that resembles a boundary.
+                 */
+                if (msr->mpd->buf_contains_line) {
+                    int i, len = (MULTIPART_BUF_SIZE - msr->mpd->bufleft);
+                    char *p = msr->mpd->buf;
+
+                    for(i = 0; i < len; i++) {
+                        if ((p[i] == '-')&&(i + 1 < len)&&(p[i + 1] == '-')) {
+                            if (strncmp(p + i + 2, msr->mpd->boundary, strlen(msr->mpd->boundary)) == 0) {
+                                msr->mpd->flag_unmatched_boundary = 1;
+                                break;
+                            }
+                        }
+                    }
                 }
             }
-            else {
+
+            /* Process as data if it was not a boundary. */
+            if (processed_as_boundary == 0) {
                 if (msr->mpd->mpp == NULL) {
+                    msr->mpd->flag_data_before = 1;
                     msr_log(msr, 4, "Multipart: Ignoring data before first boundary.");
                 } else {
                     if (msr->mpd->mpp_state == 0) {
@@ -645,14 +962,22 @@ int multipart_process_chunk(modsec_rec *msr, const char *buf,
                             /* part header lines must be shorter than
                              * MULTIPART_BUF_SIZE bytes
                              */
+                            msr->mpd->flag_error = 1;
                             *error_msg = apr_psprintf(msr->mp,
                                 "Multipart: Part header line over %i bytes long",
                                 MULTIPART_BUF_SIZE);
                             return -1;
                         }
-                        if (multipart_process_part_header(msr, error_msg) < 0) return -1;
+
+                        if (multipart_process_part_header(msr, error_msg) < 0) {
+                            msr->mpd->flag_error = 1;
+                            return -1;
+                        }
                     } else {
-                        if (multipart_process_part_data(msr, error_msg) < 0) return -1;
+                        if (multipart_process_part_data(msr, error_msg) < 0) {
+                            msr->mpd->flag_error = 1;
+                            return -1;
+                        }
                     }
                 }
             }
@@ -672,6 +997,7 @@ int multipart_process_chunk(modsec_rec *msr, const char *buf,
         }
         
         if ((msr->mpd->is_complete)&&(inleft != 0)) {
+            msr->mpd->flag_data_after = 1;
             msr_log(msr, 4, "Multipart: Ignoring data after last boundary (%i bytes left)", inleft);
             return 1;
         }
@@ -787,7 +1113,7 @@ int multipart_get_arguments(modsec_rec *msr, char *origin, apr_table_t *argument
             if (arg == NULL) return -1;
 
             arg->name = parts[i]->name;
-            arg->name_len = strlen(parts[i]->name); // TODO
+            arg->name_len = strlen(parts[i]->name);
             arg->value = parts[i]->value;
             arg->value_len = parts[i]->length;
             arg->value_origin_offset = parts[i]->offset;
@@ -812,8 +1138,6 @@ char *multipart_reconstruct_urlencoded_body_sanitise(modsec_rec *msr) {
     
     if (msr->mpd == NULL) return NULL;
 
-    // TODO Cache this data somewhere in the structure
-
     /* calculate the size of the buffer */
     body_len = 1;
     parts = (multipart_part **)msr->mpd->parts->elts;
