@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: msc_multipart.h,v 1.1.1.1 2006/10/14 09:30:43 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #ifndef _MSC_MULTIPART_H_
@@ -63,9 +61,10 @@ struct multipart_data {
     apr_array_header_t      *parts;
 
     /* mime boundary used to detect when
-     * parts end and new begin
+     * parts end and begin
      */
     char                    *boundary;
+    int                      boundary_count;
 
     /* internal buffer and other variables
      * used while parsing
@@ -86,7 +85,7 @@ struct multipart_data {
     /* part parsing state; 0 means we are reading
      * headers, 1 means we are collecting data
      */
-    int                     mpp_state;
+    int                      mpp_state;
 
     /* because of the way this parsing algorithm
      * works we hold back the last two bytes of
@@ -95,10 +94,21 @@ struct multipart_data {
      * a boundary; the first byte is an indicator
      * 0 - no content, 1 - two data bytes available
      */
-    char                    reserve[4];
-
-    int                     seen_data;    
-    int                     is_complete;
+    char                     reserve[4];
+ 
+    int                      seen_data;    
+    int                      is_complete;
+ 
+    int                      flag_error;
+    int                      flag_data_before;
+    int                      flag_data_after;
+    int                      flag_header_folding;
+    int                      flag_boundary_quoted;
+    int                      flag_lf_line;
+    int                      flag_crlf_line;
+    int                      flag_unmatched_boundary;
+    int                      flag_boundary_whitespace;
+    int                      flag_missing_semicolon;
 };
 
 
