@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: msc_reqbody.c,v 1.2 2006/12/04 21:54:10 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #include "modsecurity.h"
@@ -242,7 +240,7 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
             if (multipart_process_chunk(msr, data, length, &my_error_msg) < 0) {
                 msr->msc_reqbody_error = 1;
                 msr->msc_reqbody_error_msg = my_error_msg;
-                msr_log(msr, 4, "%s", my_error_msg);
+                msr_log(msr, 1, "Request body processor error: %s", my_error_msg);
             }
         }
         #ifdef WITH_LIBXML2
@@ -251,7 +249,7 @@ apr_status_t modsecurity_request_body_store(modsec_rec *msr,
             if (xml_process_chunk(msr, data, length, &my_error_msg) < 0) {
                 msr->msc_reqbody_error = 1;
                 msr->msc_reqbody_error_msg = my_error_msg;
-                msr_log(msr, 4, "%s", my_error_msg);
+                msr_log(msr, 1, "Request body processor error: %s", my_error_msg);
             }
         }
         #endif
