@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: re.c,v 1.15 2006/12/29 10:44:25 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #include <ctype.h>
@@ -1166,7 +1164,7 @@ static void msre_perform_disruptive_actions(modsec_rec *msr, msre_rule *rule,
 static int execute_operator(msre_var *var, msre_rule *rule, modsec_rec *msr,
     msre_actionset *acting_actionset, apr_pool_t *mptmp)
 {
-    apr_time_t time_before_regex;
+    apr_time_t time_before_regex = 0;
     char *my_error_msg = NULL;
     int rc;
 
@@ -1181,8 +1179,12 @@ static int execute_operator(msre_var *var, msre_rule *rule, modsec_rec *msr,
             var->value_len));
     }
         
-    time_before_regex = apr_time_now(); /* IMP1 time_before_regex? */
+    if (msr->txcfg->debuglog_level >= 4) {
+        time_before_regex = apr_time_now();
+    }
+
     rc = rule->op_metadata->execute(msr, rule, var, &my_error_msg);
+
     if (msr->txcfg->debuglog_level >= 4) {
         msr_log(msr, 4, "Operator completed in %" APR_TIME_T_FMT " usec.",
             (apr_time_now() - time_before_regex));
@@ -1249,12 +1251,16 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
         multi_match = 1;
     }
 
-    /* Create a memory pool that will be used during the
-     * processing of this rule only.
-     */
-    /* IMP1 Why not have one pool and just clear it between rules? */
-    if (apr_pool_create(&mptmp, NULL) != APR_SUCCESS) {
-        return -1;
+    /* Use a fresh memory sub-pool for processing each rule */
+    if (msr->msc_rule_mptmp == NULL) {
+        if (apr_pool_create(&msr->msc_rule_mptmp, msr->mp) != APR_SUCCESS) {
+            return -1;
+        }
+        mptmp = msr->msc_rule_mptmp;
+    }
+    else {
+        mptmp = msr->msc_rule_mptmp;
+        apr_pool_clear(mptmp);
     }
 
     tartab = apr_table_make(mptmp, 24);
@@ -1373,7 +1379,6 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
                     rc = execute_operator(var, rule, msr, acting_actionset, mptmp);
 
                     if (rc < 0) {
-                        apr_pool_destroy(mptmp);
                         return -1;
                     }
 
@@ -1397,7 +1402,6 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
                 rc = metadata->execute(mptmp, (unsigned char *)var->value, var->value_len,
                     &rval, &rval_length);
                 if (rc < 0) {
-                    apr_pool_destroy(mptmp);
                     return -1;
                 }
 
@@ -1423,7 +1427,6 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
             rc = execute_operator(var, rule, msr, acting_actionset, mptmp);
 
             if (rc < 0) {
-                apr_pool_destroy(mptmp);
                 return -1;
             }
 
@@ -1441,7 +1444,6 @@ apr_status_t msre_rule_process(msre_rule *rule, modsec_rec *msr) {
         }
     }
 
-    apr_pool_destroy(mptmp);
 
     return (match_count ? RULE_MATCH : RULE_NO_MATCH);
 }
