@@ -1,13 +1,11 @@
 /*
  * ModSecurity for Apache 2.x, http://www.modsecurity.org/
- * Copyright (c) 2004-2006 Thinking Stone (http://www.thinkingstone.com)
- *
- * $Id: re_variables.c,v 1.7 2007/01/23 16:08:15 ivanr Exp $
+ * Copyright (c) 2004-2007 Breach Security, Inc. (http://www.breach.com/)
  *
  * You should have received a copy of the licence along with this
  * program (stored in the file "LICENSE"). If the file is missing,
  * or if you have any other questions related to the licence, please
- * write to Thinking Stone at contact@thinkingstone.com.
+ * write to Breach Security, Inc. at support@breach.com.
  *
  */
 #include "http_core.h"
@@ -1434,7 +1432,7 @@ static int var_script_filename_generate(modsec_rec *msr, msre_var *var, msre_rul
 static int var_script_gid_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
     apr_table_t *vartab, apr_pool_t *mptmp)
 {
-    char *value = apr_psprintf(mptmp, "%i", msr->r->finfo.group);
+    char *value = apr_psprintf(mptmp, "%d", msr->r->finfo.group);
     return var_simple_generate(var, vartab, mptmp, value);
 }
 
@@ -1464,7 +1462,7 @@ static int var_script_mode_generate(modsec_rec *msr, msre_var *var, msre_rule *r
 static int var_script_uid_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
     apr_table_t *vartab, apr_pool_t *mptmp)
 {
-    char *value = apr_psprintf(mptmp, "%i", msr->r->finfo.user);
+    char *value = apr_psprintf(mptmp, "%d", msr->r->finfo.user);
     return var_simple_generate(var, vartab, mptmp, value);
 }
 
@@ -1653,6 +1651,149 @@ static int var_webappid_generate(modsec_rec *msr, msre_var *var, msre_rule *rule
     return var_simple_generate(var, vartab, mptmp, value);
 }
 
+/* MULTIPART_BOUNDARY_QUOTED */
+
+static int var_multipart_boundary_quoted_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_boundary_quoted != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_BOUNDARY_WHITESPACE */
+
+static int var_multipart_boundary_whitespace_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_boundary_whitespace != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_DATA_AFTER */
+
+static int var_multipart_data_after_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_data_after != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_DATA_BEFORE */
+
+static int var_multipart_data_before_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_data_before != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_HEADER_FOLDING */
+
+static int var_multipart_header_folding_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_header_folding != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_CRLF_LINE */
+
+static int var_multipart_crlf_line_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_crlf_line != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_CRLF_LF_LINES */
+
+static int var_multipart_crlf_lf_lines_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_lf_line != 0)&&(msr->mpd->flag_crlf_line != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_LF_LINE */
+
+static int var_multipart_lf_line_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_lf_line != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_MISSING_SEMICOLON */
+
+static int var_multipart_missing_semicolon_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_missing_semicolon != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
+/* MULTIPART_STRICT_ERROR */
+
+static int var_multipart_strict_error_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if (msr->mpd != NULL) {
+        /* Respond positive if at least one of the multipart flags is raised. */
+        if (  (msr->mpd->flag_error)
+            ||(msr->mpd->flag_boundary_quoted != 0)
+            ||(msr->mpd->flag_boundary_whitespace != 0)
+            ||(msr->mpd->flag_data_before != 0)
+            ||(msr->mpd->flag_data_after != 0)
+            ||(msr->mpd->flag_header_folding != 0)
+            ||(msr->mpd->flag_lf_line != 0)
+            ||(msr->mpd->flag_missing_semicolon != 0)
+        ) {
+            return var_simple_generate(var, vartab, mptmp, "1");
+        }
+    }
+
+    return var_simple_generate(var, vartab, mptmp, "0");
+}
+
+/* MULTIPART_UNMATCHED_BOUNDARY */
+
+static int var_multipart_unmatched_boundary_generate(modsec_rec *msr, msre_var *var, msre_rule *rule,
+    apr_table_t *vartab, apr_pool_t *mptmp)
+{
+    if ((msr->mpd != NULL)&&(msr->mpd->flag_unmatched_boundary != 0)) {
+        return var_simple_generate(var, vartab, mptmp, "1");
+    } else {
+        return var_simple_generate(var, vartab, mptmp, "0");
+    }
+}
+
 /* ---------------------------------------------- */
 
 /**
@@ -2433,4 +2574,127 @@ void msre_engine_register_default_variables(msre_engine *engine) {
         VAR_DONT_CACHE,
         PHASE_RESPONSE_HEADERS
     );
+
+    /* MULTIPART_BOUNDARY_QUOTED */
+    msre_engine_variable_register(engine,
+        "MULTIPART_BOUNDARY_QUOTED",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_boundary_quoted_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_BOUNDARY_WHITESPACE */
+    msre_engine_variable_register(engine,
+        "MULTIPART_BOUNDARY_WHITESPACE",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_boundary_whitespace_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_DATA_AFTER */
+    msre_engine_variable_register(engine,
+        "MULTIPART_DATA_AFTER",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_data_after_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_DATA_BEFORE */
+    msre_engine_variable_register(engine,
+        "MULTIPART_DATA_BEFORE",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_data_before_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_HEADER_FOLDING */
+    msre_engine_variable_register(engine,
+        "MULTIPART_HEADER_FOLDING",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_header_folding_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_CRLF_LINE */
+    msre_engine_variable_register(engine,
+        "MULTIPART_CRLF_LINE",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_crlf_line_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_CRLF_LF_LINES */
+    msre_engine_variable_register(engine,
+        "MULTIPART_CRLF_LF_LINES",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_crlf_lf_lines_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+
+    /* MULTIPART_LF_LINE */
+    msre_engine_variable_register(engine,
+        "MULTIPART_LF_LINE",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_lf_line_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_MISSING_SEMICOLON */
+    msre_engine_variable_register(engine,
+        "MULTIPART_MISSING_SEMICOLON",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_missing_semicolon_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+
+    /* MULTIPART_STRICT_ERROR */
+    msre_engine_variable_register(engine,
+        "MULTIPART_STRICT_ERROR",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_strict_error_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
+
+    /* MULTIPART_UNMATCHED_BOUNDARY */
+    msre_engine_variable_register(engine,
+        "MULTIPART_UNMATCHED_BOUNDARY",
+        VAR_SIMPLE,
+        0, 0,
+        NULL,
+        var_multipart_unmatched_boundary_generate,
+        VAR_CACHE,
+        PHASE_REQUEST_BODY
+    );
 }
